@@ -64,7 +64,7 @@
 #endif
 
 /* detect 64 bit platform */
-#if defined(__X86_64__) || defined(__aarch64__)
+#if defined(__X86_64__)
 #define __64BIT__
 #endif
 
