@@ -344,7 +344,7 @@ namespace embree
 
     return cpu_features;
 
-#elif defined(__ARM_NEON) || defined(__EMSCRIPTEN__)
+#elif defined(__ARM_NEON)
 
     int cpu_features = CPU_FEATURE_NEON|CPU_FEATURE_SSE|CPU_FEATURE_SSE2;
     cpu_features |= CPU_FEATURE_SSE3|CPU_FEATURE_SSSE3|CPU_FEATURE_SSE42;
@@ -462,35 +462,6 @@ namespace embree
     return std::string(filename);
   }
 
-  unsigned int getNumberOfLogicalThreads() 
-  {
-    static int nThreads = -1;
-    if (nThreads != -1) return nThreads;
-
-    typedef WORD (WINAPI *GetActiveProcessorGroupCountFunc)();
-    typedef DWORD (WINAPI *GetActiveProcessorCountFunc)(WORD);
-    HMODULE hlib = LoadLibrary("Kernel32");
-    GetActiveProcessorGroupCountFunc pGetActiveProcessorGroupCount = (GetActiveProcessorGroupCountFunc)GetProcAddress(hlib, "GetActiveProcessorGroupCount");
-    GetActiveProcessorCountFunc      pGetActiveProcessorCount      = (GetActiveProcessorCountFunc)     GetProcAddress(hlib, "GetActiveProcessorCount");
-
-    if (pGetActiveProcessorGroupCount && pGetActiveProcessorCount) 
-    {
-      int groups = pGetActiveProcessorGroupCount();
-      int totalProcessors = 0;
-      for (int i = 0; i < groups; i++) 
-        totalProcessors += pGetActiveProcessorCount(i);
-      nThreads = totalProcessors;
-    }
-    else
-    {
-      SYSTEM_INFO sysinfo;
-      GetSystemInfo(&sysinfo);
-      nThreads = sysinfo.dwNumberOfProcessors;
-    }
-    assert(nThreads);
-    return nThreads;
-  }
-
   int getTerminalWidth() 
   {
     HANDLE handle = GetStdHandle(STD_OUTPUT_HANDLE);
@@ -512,20 +483,6 @@ namespace embree
   void sleepSeconds(double t) {
     Sleep(DWORD(1000.0*t));
   }
-
-  size_t getVirtualMemoryBytes()
-  {
-    PROCESS_MEMORY_COUNTERS info;
-    GetProcessMemoryInfo( GetCurrentProcess( ), &info, sizeof(info) );
-    return (size_t)info.QuotaPeakPagedPoolUsage;
-  }
-
-  size_t getResidentMemoryBytes()
-  {
-    PROCESS_MEMORY_COUNTERS info;
-    GetProcessMemoryInfo( GetCurrentProcess( ), &info, sizeof(info) );
-    return (size_t)info.WorkingSetSize;
-  }
 }
 #endif
 
@@ -549,22 +506,6 @@ namespace embree
       return std::string();
     return std::string(buf);
   }
-
-  size_t getVirtualMemoryBytes()
-  {
-    size_t virt, resident, shared;
-    std::ifstream buffer("/proc/self/statm");
-    buffer >> virt >> resident >> shared;
-    return virt*sysconf(_SC_PAGE_SIZE);
-  }
-
-  size_t getResidentMemoryBytes()
-  {
-    size_t virt, resident, shared;
-    std::ifstream buffer("/proc/self/statm");
-    buffer >> virt >> resident >> shared;
-    return resident*sysconf(_SC_PAGE_SIZE);
-  }
 }
 
 #endif
@@ -589,14 +530,6 @@ namespace embree
       return std::string();
     return std::string(buf);
   }
-
-  size_t getVirtualMemoryBytes() {
-    return 0;
-  }
-   
-  size_t getResidentMemoryBytes() {
-    return 0;
-  }
 }
 
 #endif
@@ -619,14 +552,6 @@ namespace embree
       return std::string();
     return std::string(buf);
   }
-
-  size_t getVirtualMemoryBytes() {
-    return 0;
-  }
-   
-  size_t getResidentMemoryBytes() {
-    return 0;
-  }
 }
 
 #endif
@@ -642,46 +567,8 @@ namespace embree
 #include <sys/time.h>
 #include <pthread.h>
 
-#if defined(__EMSCRIPTEN__)
-#include <emscripten.h>
-#endif
-
 namespace embree
 {
-  unsigned int getNumberOfLogicalThreads() 
-  {
-    static int nThreads = -1;
-    if (nThreads != -1) return nThreads;
-
-#if defined(__MACOSX__) || defined(__ANDROID__)
-    nThreads = sysconf(_SC_NPROCESSORS_ONLN); // does not work in Linux LXC container
-    assert(nThreads);
-#elif defined(__EMSCRIPTEN__)
-    // WebAssembly supports pthreads, but not pthread_getaffinity_np. Get the number of logical
-    // threads from the browser or Node.js using JavaScript.
-    nThreads = MAIN_THREAD_EM_ASM_INT({
-        const isBrowser = typeof window !== 'undefined';
-        const isNode = typeof process !== 'undefined' && process.versions != null &&
-            process.versions.node != null;
-        if (isBrowser) {
-            // Return 1 if the browser does not expose hardwareConcurrency.
-            return window.navigator.hardwareConcurrency || 1;
-        } else if (isNode) {
-            return require('os').cpus().length;
-        } else {
-            return 1;
-        }
-    });
-#else
-    cpu_set_t set;
-    if (pthread_getaffinity_np(pthread_self(), sizeof(set), &set) == 0)
-      nThreads = CPU_COUNT(&set);
-#endif
-    
-    assert(nThreads);
-    return nThreads;
-  }
-
   int getTerminalWidth() 
   {
     struct winsize info;
