@@ -11,12 +11,32 @@
 #include <vector>
 #include <iostream>
 #include <fstream>
+#include <malloc.h>
 
 void* dispatchGlobalsPtr = nullptr;
 
 static uint32_t global_width = 512;
 static uint32_t global_height = 512;
 
+static bool use_instance = false;
+static bool use_device_memory = false;
+
+#if defined(_WIN32)
+  void* my_aligned_malloc(size_t size, size_t align) {
+    return _aligned_malloc(size,align);
+  }
+  void my_aligned_free(void* ptr) {
+    _aligned_free(ptr);
+  }
+#else
+  void* my_aligned_malloc(size_t size, size_t align) {
+    return aligned_alloc(align,size);
+  }
+  void my_aligned_free(void* ptr) {
+    free(ptr);
+  }
+#endif
+
 void exception_handler(sycl::exception_list exceptions)
 {
   for (std::exception_ptr const& e : exceptions) {
@@ -101,10 +121,9 @@ size_t compareTga(const std::string& fileNameA, const std::string& fileNameB)
   return diff;
 }
 
-/* Properly allocates an acceleration structure buffer using ze_raytracing_mem_alloc_ext_desc_t property. */
-void* alloc_accel_buffer(size_t bytes, sycl::device device, sycl::context context)
+/* get RTAS buffer alignment requirement */
+size_t accel_buffer_alignment(sycl::device device)
 {
-  ze_context_handle_t hContext = sycl::get_native<sycl::backend::ext_oneapi_level_zero>(context);
   ze_device_handle_t  hDevice  = sycl::get_native<sycl::backend::ext_oneapi_level_zero>(device);
   
   ze_rtas_device_exp_properties_t rtasProp = { ZE_STRUCTURE_TYPE_RTAS_DEVICE_EXP_PROPERTIES };
@@ -112,7 +131,16 @@ void* alloc_accel_buffer(size_t bytes, sycl::device device, sycl::context contex
   ze_result_t err = ZeWrapper::zeDeviceGetProperties(hDevice, &devProp );
   if (err != ZE_RESULT_SUCCESS)
     throw std::runtime_error("zeDeviceGetProperties failed");
-  
+
+  return rtasProp.rtasBufferAlignment;
+}
+
+/* Properly allocates a USM acceleration structure buffer using ze_raytracing_mem_alloc_ext_desc_t property. */
+void* alloc_shared_accel_buffer(size_t bytes, sycl::device device, sycl::context context)
+{
+  ze_context_handle_t hContext = sycl::get_native<sycl::backend::ext_oneapi_level_zero>(context);
+  ze_device_handle_t  hDevice  = sycl::get_native<sycl::backend::ext_oneapi_level_zero>(device);
+
   ze_raytracing_mem_alloc_ext_desc_t rt_desc;
   rt_desc.stype = ZE_STRUCTURE_TYPE_RAYTRACING_MEM_ALLOC_EXT_DESC;
   rt_desc.pNext = nullptr;
@@ -128,9 +156,38 @@ void* alloc_accel_buffer(size_t bytes, sycl::device device, sycl::context contex
   host_desc.stype = ZE_STRUCTURE_TYPE_HOST_MEM_ALLOC_DESC;
   host_desc.pNext = nullptr;
   host_desc.flags = ZE_HOST_MEM_ALLOC_FLAG_BIAS_CACHED;
-  
+
+  const size_t rtasAlignment = accel_buffer_alignment(device);
+ 
   void* ptr = nullptr;
-  ze_result_t result = ZeWrapper::zeMemAllocShared(hContext,&device_desc,&host_desc,bytes,rtasProp.rtasBufferAlignment,hDevice,&ptr);
+  ze_result_t result = ZeWrapper::zeMemAllocShared(hContext,&device_desc,&host_desc,bytes,rtasAlignment,hDevice,&ptr);
+  if (result != ZE_RESULT_SUCCESS)
+    throw std::runtime_error("acceleration buffer allocation failed");
+
+  return ptr;
+}
+
+/* Properly allocates a device memory acceleration structure buffer using ze_raytracing_mem_alloc_ext_desc_t property. */
+void* alloc_device_accel_buffer(size_t bytes, sycl::device device, sycl::context context)
+{
+  ze_context_handle_t hContext = sycl::get_native<sycl::backend::ext_oneapi_level_zero>(context);
+  ze_device_handle_t  hDevice  = sycl::get_native<sycl::backend::ext_oneapi_level_zero>(device);
+  
+  ze_raytracing_mem_alloc_ext_desc_t rt_desc;
+  rt_desc.stype = ZE_STRUCTURE_TYPE_RAYTRACING_MEM_ALLOC_EXT_DESC;
+  rt_desc.pNext = nullptr;
+  rt_desc.flags = 0;
+    
+  ze_device_mem_alloc_desc_t device_desc;
+  device_desc.stype = ZE_STRUCTURE_TYPE_DEVICE_MEM_ALLOC_DESC;
+  device_desc.pNext = &rt_desc;
+  device_desc.flags = ZE_DEVICE_MEM_ALLOC_FLAG_BIAS_CACHED;
+  device_desc.ordinal = 0;
+
+  const size_t rtasAlignment = accel_buffer_alignment(device);
+ 
+  void* ptr = nullptr;
+  ze_result_t result = ZeWrapper::zeMemAllocDevice(hContext,&device_desc,bytes,rtasAlignment,hDevice,&ptr);
   if (result != ZE_RESULT_SUCCESS)
     throw std::runtime_error("acceleration buffer allocation failed");
 
@@ -173,7 +230,7 @@ void* allocDispatchGlobals(sycl::device device, sycl::context context)
   size_t num_rtstacks = 1<<17; // this is sufficiently large also for PVC
   size_t dispatchGlobalSize = 128+num_rtstacks*rtstack_bytes;
   
-  void* dispatchGlobalsPtr = alloc_accel_buffer(dispatchGlobalSize,device,context);
+  void* dispatchGlobalsPtr = alloc_shared_accel_buffer(dispatchGlobalSize,device,context);
   memset(dispatchGlobalsPtr, 0, dispatchGlobalSize);
 
   DispatchGlobals* dg = (DispatchGlobals*) dispatchGlobalsPtr;
@@ -300,11 +357,13 @@ ze_rtas_float3_exp_t vertices[] = {
 };
 
 /* builds acceleration structure */
-void* build_rtas(sycl::device device, sycl::context context)
+void* build_rtas(sycl::device device, sycl::context context, sycl::queue queue, std::vector<ze_rtas_builder_geometry_info_exp_t*>& descs)
 {
   /* get L0 handles */
   ze_driver_handle_t hDriver = sycl::get_native<sycl::backend::ext_oneapi_level_zero>(device.get_platform());
   ze_device_handle_t hDevice = sycl::get_native<sycl::backend::ext_oneapi_level_zero>(device);
+  ze_context_handle_t hContext = sycl::get_native<sycl::backend::ext_oneapi_level_zero>(context);
+  auto hQueue = sycl::get_native<sycl::backend::ext_oneapi_level_zero>(queue);
     
   /* create rtas builder object */
   ze_rtas_builder_exp_desc_t builderDesc = { ZE_STRUCTURE_TYPE_RTAS_BUILDER_EXP_DESC };
@@ -313,26 +372,6 @@ void* build_rtas(sycl::device device, sycl::context context)
   if (err != ZE_RESULT_SUCCESS)
     throw std::runtime_error("ze_rtas_builder creation failed");
     
-  /* create geometry descriptor for single triangle mesh */
-  ze_rtas_builder_triangles_geometry_info_exp_t mesh = {};
-  mesh.geometryType = ZE_RTAS_BUILDER_GEOMETRY_TYPE_EXP_TRIANGLES;
-  mesh.geometryFlags = 0;
-  mesh.geometryMask = 0xFF;
-  
-  mesh.triangleFormat = ZE_RTAS_BUILDER_INPUT_DATA_FORMAT_EXP_TRIANGLE_INDICES_UINT32;
-  mesh.triangleCount = sizeof(indices)/sizeof(ze_rtas_triangle_indices_uint32_exp_t);
-  mesh.triangleStride = sizeof(ze_rtas_triangle_indices_uint32_exp_t);
-  mesh.pTriangleBuffer = indices;
-
-  mesh.vertexFormat = ZE_RTAS_BUILDER_INPUT_DATA_FORMAT_EXP_FLOAT3;
-  mesh.vertexCount = sizeof(vertices)/sizeof(ze_rtas_float3_exp_t);
-  mesh.vertexStride = sizeof(ze_rtas_float3_exp_t);
-  mesh.pVertexBuffer = vertices;
-
-  /* fill geometry descriptor array with pointer to single geometry descriptor */
-  std::vector<ze_rtas_builder_geometry_info_exp_t*> descs;
-  descs.push_back((ze_rtas_builder_geometry_info_exp_t*)&mesh);
-  
   /* get acceleration structure format for this device */
   ze_rtas_device_exp_properties_t rtasProp = { ZE_STRUCTURE_TYPE_RTAS_DEVICE_EXP_PROPERTIES };
   ze_device_properties_t devProp = { ZE_STRUCTURE_TYPE_DEVICE_PROPERTIES, &rtasProp };
@@ -360,7 +399,7 @@ void* build_rtas(sycl::device device, sycl::context context)
 
   /* just for debugging purposes */
 #if defined(EMBREE_SYCL_ALLOC_DISPATCH_GLOBALS)
-  ze_rtas_builder_build_op_debug_exp_desc_t buildOpDebug = { ZE_STRUCTURE_TYPE_RTAS_BUILDER_BUILD_OP_DEBUG_EXP_DESC };
+  ze_rtas_builder_build_op_debug_desc_t buildOpDebug = { ZE_STRUCTURE_TYPE_RTAS_BUILDER_BUILD_OP_DEBUG_DESC };
   buildOpDebug.dispatchGlobalsPtr = dispatchGlobalsPtr;
   buildOp.pNext = &buildOpDebug;
 #endif
@@ -377,13 +416,23 @@ void* build_rtas(sycl::device device, sycl::context context)
 
   /* allocate acceleration structure buffer */
   size_t accelBytes = buildProps.rtasBufferSizeBytesMaxRequired;
-  void* accel = alloc_accel_buffer(accelBytes,device,context);
-  memset(accel,0,accelBytes); // optional
+  void* accelHost = nullptr;
+  void* accelDevice = nullptr;
+  if (use_device_memory) {
+    const size_t rtasAlignment = accel_buffer_alignment(device);
+    accelHost = my_aligned_malloc(accelBytes,rtasAlignment);
+    accelDevice = alloc_device_accel_buffer(accelBytes,device,context);
+  }
+  else {
+    accelHost = alloc_shared_accel_buffer(accelBytes,device,context);
+    accelDevice = accelHost;
+  }
+  memset(accelHost,0,accelBytes); // optional
   
   /* build acceleration strucuture multi threaded */
   err = ZeWrapper::zeRTASBuilderBuildExp(hBuilder,&buildOp,
                                   scratchBuffer.data(),scratchBuffer.size(),
-                                  accel, accelBytes,
+                                  accelHost, accelBytes,
                                   hParallelOperation,
                                   nullptr, &bounds, &accelBufferBytesOut);
   
@@ -415,7 +464,139 @@ void* build_rtas(sycl::device device, sycl::context context)
   if (err != ZE_RESULT_SUCCESS)
     throw std::runtime_error("zeRTASBuilderDestroyExp failed");
   
-  return accel;
+  /* copy host buffer to device memory */
+  if (use_device_memory)
+  {
+    sycl::event event = sycl::event();
+    ze_event_handle_t hEvent = sycl::get_native<sycl::backend::ext_oneapi_level_zero>(event);
+
+    /* command list codepath */
+    if (std::holds_alternative<ze_command_list_handle_t>(hQueue))
+    {
+       std::cout << "using ze_command_list_handle_t to copy RTAS" << std::endl;
+       
+      ze_command_list_handle_t hCommandList = std::get<ze_command_list_handle_t>(hQueue);
+      
+      err = ZeWrapper::zeRTASBuilderCommandListAppendCopyExp(hCommandList, accelDevice, accelHost, accelBytes, hEvent, 0, nullptr);
+      if (err != ZE_RESULT_SUCCESS)
+        throw std::runtime_error("zeRTASBuilderCommandListAppendCopyExp failed");
+
+      /* this is required for some reason */
+      queue.submit([&](sycl::handler& cgh) {
+        cgh.single_task([=]() {});
+      });
+
+      queue.wait();
+    }
+
+    /* command queue codepath */
+    else
+    {
+      std::cout << "using ze_command_queue_handle_t to copy RTAS" << std::endl;
+       
+      ze_command_queue_handle_t hCommandQueue = std::get<ze_command_queue_handle_t>(hQueue);
+      
+      ze_command_list_handle_t hCommandList = nullptr;
+      ze_command_list_desc_t commandListDesc = { ZE_STRUCTURE_TYPE_COMMAND_LIST_DESC, nullptr, 0, ZE_COMMAND_QUEUE_FLAG_IN_ORDER };
+      err = ZeWrapper::zeCommandListCreate(hContext,hDevice,&commandListDesc,&hCommandList);
+      if (err != ZE_RESULT_SUCCESS)
+        throw std::runtime_error("zeCommandListCreate failed");
+      
+      err = ZeWrapper::zeRTASBuilderCommandListAppendCopyExp(hCommandList, accelDevice, accelHost, accelBytes, hEvent, 0, nullptr);
+      if (err != ZE_RESULT_SUCCESS)
+        throw std::runtime_error("zeRTASBuilderCommandListAppendCopyExp failed");
+      
+      err = ZeWrapper::zeCommandListClose(hCommandList);
+      if (err != ZE_RESULT_SUCCESS)
+        throw std::runtime_error("zeCommandListClose failed");
+      
+      err = ZeWrapper::zeCommandQueueExecuteCommandLists(hCommandQueue, 1, &hCommandList, nullptr);
+      if (err != ZE_RESULT_SUCCESS)
+        throw std::runtime_error("zeCommandQueueExecuteCommandLists failed");
+      
+      queue.wait();
+      
+      err = ZeWrapper::zeCommandListDestroy(hCommandList);
+      if (err != ZE_RESULT_SUCCESS)
+        throw std::runtime_error("zeCommandListDestroy failed");
+    }
+    my_aligned_free(accelHost);
+  }
+  
+  return accelDevice;
+}
+
+/* builds bottom level acceleration structure */
+void* build_blas(sycl::device device, sycl::context context, sycl::queue queue)
+{
+  /* create geometry descriptor for single triangle mesh */
+  ze_rtas_builder_triangles_geometry_info_exp_t mesh = {};
+  mesh.geometryType = ZE_RTAS_BUILDER_GEOMETRY_TYPE_EXP_TRIANGLES;
+  mesh.geometryFlags = 0;
+  mesh.geometryMask = 0xFF;
+  
+  mesh.triangleFormat = ZE_RTAS_BUILDER_INPUT_DATA_FORMAT_EXP_TRIANGLE_INDICES_UINT32;
+  mesh.triangleCount = sizeof(indices)/sizeof(ze_rtas_triangle_indices_uint32_exp_t);
+  mesh.triangleStride = sizeof(ze_rtas_triangle_indices_uint32_exp_t);
+  mesh.pTriangleBuffer = indices;
+
+  mesh.vertexFormat = ZE_RTAS_BUILDER_INPUT_DATA_FORMAT_EXP_FLOAT3;
+  mesh.vertexCount = sizeof(vertices)/sizeof(ze_rtas_float3_exp_t);
+  mesh.vertexStride = sizeof(ze_rtas_float3_exp_t);
+  mesh.pVertexBuffer = vertices;
+
+  /* fill geometry descriptor array with pointer to single geometry descriptor */
+  std::vector<ze_rtas_builder_geometry_info_exp_t*> descs;
+  descs.push_back((ze_rtas_builder_geometry_info_exp_t*)&mesh);
+
+  /* build RTAS from descriptors */
+  return build_rtas(device,context,queue,descs);
+}
+
+/* builds top level acceleration structure */
+void* build_tlas(sycl::device device, sycl::context context, sycl::queue queue, void* blas)
+{
+  /* create identity transform */
+  ze_rtas_transform_float3x4_aligned_column_major_exp_t xfmdata;
+  xfmdata.vx_x = 1.0f;
+  xfmdata.vx_y = 0.0f;
+  xfmdata.vx_z = 0.0f;
+  xfmdata.pad0 = 0.0f;
+  xfmdata.vy_x = 0.0f;
+  xfmdata.vy_y = 1.0f;
+  xfmdata.vy_z = 0.0f;
+  xfmdata.pad1 = 0.0f;
+  xfmdata.vz_x = 0.0f;
+  xfmdata.vz_y = 0.0f;
+  xfmdata.vz_z = 1.0f;
+  xfmdata.pad2 = 0.0f;
+  xfmdata.p_x  = 0.0f;
+  xfmdata.p_y  = 0.0f;
+  xfmdata.p_z  = 0.0f;
+  xfmdata.pad3  = 0.0f;
+
+  /* conservative bounds for model */
+  ze_rtas_aabb_exp_t bounds = {
+    { 0,0,0 }, { 1000, 1000, 1000 }
+  };
+  
+  /* create geometry descriptor for instance */
+  ze_rtas_builder_instance_geometry_info_exp_t inst = {};
+  inst.geometryType = ZE_RTAS_BUILDER_GEOMETRY_TYPE_EXP_INSTANCE;
+  inst.instanceFlags = 0;
+  inst.geometryMask = 0xFF;
+  inst.instanceUserID = 0;
+  inst.transformFormat = ZE_RTAS_BUILDER_INPUT_DATA_FORMAT_EXP_FLOAT3X4_ALIGNED_COLUMN_MAJOR;
+  inst.pTransform = (float*)&xfmdata;
+  inst.pBounds = &bounds;
+  inst.pAccelerationStructure = blas;
+
+  /* fill geometry descriptor array with pointer to single geometry descriptor */
+  std::vector<ze_rtas_builder_geometry_info_exp_t*> descs;
+  descs.push_back((ze_rtas_builder_geometry_info_exp_t*)&inst);
+
+  /* build RTAS from descriptors */
+  return build_rtas(device,context,queue,descs);
 }
 
 /* render using simple UV shading */
@@ -467,12 +648,7 @@ void render(unsigned int x, unsigned int y, void* bvh, unsigned int* pixels, uns
 
 int main(int argc, char* argv[]) try
 {
-  /* use can specify reference image to compare against */
-#if defined(EMBREE_SYCL_L0_RTAS_BUILDER)
   ZeWrapper::RTAS_BUILD_MODE rtas_build_mode = ZeWrapper::RTAS_BUILD_MODE::LEVEL_ZERO;
-#else
-  ZeWrapper::RTAS_BUILD_MODE rtas_build_mode = ZeWrapper::RTAS_BUILD_MODE::INTERNAL;
-#endif
   
   char* reference_img = NULL;
   for (int i=1; i<argc; i++)
@@ -487,8 +663,11 @@ int main(int argc, char* argv[]) try
     else if (strcmp(argv[i], "--level-zero-rtas-builder") == 0) {
       rtas_build_mode = ZeWrapper::RTAS_BUILD_MODE::LEVEL_ZERO;
     }
-    else if (strcmp(argv[i], "--default-rtas-builder") == 0) {
-      rtas_build_mode = ZeWrapper::RTAS_BUILD_MODE::AUTO;
+    else if (strcmp(argv[i], "--instance") == 0) {
+      use_instance = true;
+    }
+    else if (strcmp(argv[i], "--device-memory") == 0) {
+      use_device_memory = true;
     }
     else if (strcmp(argv[i], "--size") == 0) {
       if (++i >= argc) throw std::runtime_error("--size: width expected");
@@ -505,6 +684,11 @@ int main(int argc, char* argv[]) try
     }
   }
 
+  if (rtas_build_mode == ZeWrapper::INTERNAL)
+    std::cout << "using internal RTAS builder (ZE_experimental_rtas_builder)" << std::endl;
+  else
+    std::cout << "using Level Zero RTAS builder (ZE_experimental_rtas_builder)" << std::endl;
+
   /* create SYCL objects */
   sycl::device device = sycl::device(sycl::gpu_selector_v);
   sycl::queue queue = sycl::queue(device,exception_handler);
@@ -518,49 +702,50 @@ int main(int argc, char* argv[]) try
   ze_result_t result = ZE_RESULT_SUCCESS;
   sycl::platform platform = device.get_platform();
   ze_driver_handle_t hDriver = sycl::get_native<sycl::backend::ext_oneapi_level_zero>(platform);
+  ze_device_handle_t hDevice = sycl::get_native<sycl::backend::ext_oneapi_level_zero>(device);
 
-  /* enable RTAS extension only when enabled */
-  if (rtas_build_mode == ZeWrapper::RTAS_BUILD_MODE::AUTO)
+  /* check if ray tracing hardware is supported */
+  ze_device_raytracing_ext_properties_t raytracing_properties = { ZE_STRUCTURE_TYPE_DEVICE_RAYTRACING_EXT_PROPERTIES };
+  ze_device_module_properties_t module_properties = { ZE_STRUCTURE_TYPE_DEVICE_MODULE_PROPERTIES, &raytracing_properties };
+  result = ZeWrapper::zeDeviceGetModuleProperties(hDevice, &module_properties);
+  if (result != ZE_RESULT_SUCCESS)
+    throw std::runtime_error("zeDeviceGetModuleProperties failed");
+  
+  const bool rayQuerySupported = raytracing_properties.flags & ZE_DEVICE_RAYTRACING_EXT_FLAG_RAYQUERY;
+  if (!rayQuerySupported)
+    throw std::runtime_error("Device does not support ray tracing");
+
+  /* check if RTAS extension available */
+  uint32_t count = 0;
+  std::vector<ze_driver_extension_properties_t> extensions;
+  result = ZeWrapper::zeDriverGetExtensionProperties(hDriver,&count,extensions.data());
+  if (result != ZE_RESULT_SUCCESS)
+    throw std::runtime_error("zeDriverGetExtensionProperties failed");
+  
+  extensions.resize(count);
+  result = ZeWrapper::zeDriverGetExtensionProperties(hDriver,&count,extensions.data());
+  if (result != ZE_RESULT_SUCCESS)
+    throw std::runtime_error("zeDriverGetExtensionProperties failed");
+  
+  bool ze_rtas_builder = false;
+  for (uint32_t i=0; i<extensions.size(); i++)
   {
-    uint32_t count = 0;
-    std::vector<ze_driver_extension_properties_t> extensions;
-    result = ZeWrapper::zeDriverGetExtensionProperties(hDriver,&count,extensions.data());
-    if (result != ZE_RESULT_SUCCESS)
-      throw std::runtime_error("zeDriverGetExtensionProperties failed");
-    
-    extensions.resize(count);
-    result = ZeWrapper::zeDriverGetExtensionProperties(hDriver,&count,extensions.data());
-    if (result != ZE_RESULT_SUCCESS)
-      throw std::runtime_error("zeDriverGetExtensionProperties failed");
-    
-    bool ze_rtas_builder = false;
-    for (uint32_t i=0; i<extensions.size(); i++)
-    {
-      if (strncmp("ZE_experimental_rtas_builder",extensions[i].name,sizeof(extensions[i].name)) == 0)
-        ze_rtas_builder = true;
-    }
-
-    if (ze_rtas_builder)
-      result = ZeWrapper::initRTASBuilder(hDriver,ZeWrapper::RTAS_BUILD_MODE::AUTO);
-    else
-      result = ZeWrapper::initRTASBuilder(hDriver,ZeWrapper::RTAS_BUILD_MODE::INTERNAL);
+    if (strncmp("ZE_experimental_rtas_builder",extensions[i].name,sizeof(extensions[i].name)) == 0)
+      ze_rtas_builder = true;
   }
-  else
-    result = ZeWrapper::initRTASBuilder(hDriver,rtas_build_mode);
+  if (!ze_rtas_builder)
+    throw std::runtime_error("ZE_experimental_rtas_builder extension not available");
 
+  /* initialize RTAS builder extension in ZeWrapper */
+  result = ZeWrapper::initRTASBuilder(EXP_API,hDriver,rtas_build_mode);
+  
   if (result == ZE_RESULT_ERROR_DEPENDENCY_UNAVAILABLE)
     throw std::runtime_error("cannot load ZE_experimental_rtas_builder extension");
   
   if (result != ZE_RESULT_SUCCESS)
     throw std::runtime_error("cannot initialize ZE_experimental_rtas_builder extension");
   
-  if (ZeWrapper::rtas_builder == ZeWrapper::INTERNAL)
-    std::cout << "using internal RTAS builder" << std::endl;
-  else
-    std::cout << "using Level Zero RTAS builder" << std::endl;
-
   /* get acceleration structure format for this device */
-  ze_device_handle_t  hDevice  = sycl::get_native<sycl::backend::ext_oneapi_level_zero>(device);
   ze_rtas_device_exp_properties_t rtasProp = { ZE_STRUCTURE_TYPE_RTAS_DEVICE_EXP_PROPERTIES };
   ze_device_properties_t devProp = { ZE_STRUCTURE_TYPE_DEVICE_PROPERTIES, &rtasProp };
   result = ZeWrapper::zeDeviceGetProperties(hDevice, &devProp );
@@ -579,8 +764,12 @@ int main(int argc, char* argv[]) try
   dispatchGlobalsPtr = allocDispatchGlobals(device,context);
 #endif
 
-  /* build acceleration structure */
-  void* bvh = build_rtas(device,context);
+  /* build bottom level acceleration structure */
+  void* bvh = build_blas(device,context,queue);
+
+  /* build toplevel acceleration structure */
+  if (use_instance)
+    bvh = build_tlas(device,context,queue,bvh);
 
   /* creates framebuffer */
   const uint32_t width = global_width;
@@ -628,8 +817,8 @@ int main(int argc, char* argv[]) try
   /* compare to reference image */
   const size_t err = compareTga("cornell_box.tga", "cornell_box_reference.tga");
   std::cout << "difference to reference image is " << err << std::endl;
+  
   const bool ok = err < 32;
-  std::cout << "cornell_box ";
   if (ok) std::cout << "[PASSED]" << std::endl;
   else    std::cout << "[FAILED]" << std::endl;
 
