@@ -8,6 +8,7 @@
 #pragma clang diagnostic ignored "-Wignored-attributes"
 
 #include <sycl/sycl.hpp>
+#include "usm_allocator.h"
 #include "tbb/tbb.h"
 
 #if defined(ZE_RAYTRACING)
@@ -760,11 +761,11 @@ public:
   ze_rtas_builder_geometry_exp_flags_t gflags = 0;
   bool procedural = false;
   
-  typedef sycl::usm_allocator<sycl::int4, sycl::usm::alloc::shared> triangles_alloc_ty;
+  typedef usm_allocator<sycl::int4, sycl::usm::alloc::shared> triangles_alloc_ty;
   triangles_alloc_ty triangles_alloc;
   std::vector<sycl::int4, triangles_alloc_ty> triangles;
 
-  typedef sycl::usm_allocator<sycl::float3, sycl::usm::alloc::shared> vertices_alloc_ty;
+  typedef usm_allocator<sycl::float3, sycl::usm::alloc::shared> vertices_alloc_ty;
   vertices_alloc_ty vertices_alloc;
   std::vector<sycl::float3, vertices_alloc_ty> vertices;
   
@@ -1193,7 +1194,7 @@ struct Scene
 
     /* just for debugging purposes */
 #if defined(EMBREE_SYCL_ALLOC_DISPATCH_GLOBALS)
-    ze_rtas_builder_build_op_debug_exp_desc_t buildOpDebug = { ZE_STRUCTURE_TYPE_RTAS_BUILDER_BUILD_OP_DEBUG_EXP_DESC };
+    ze_rtas_builder_build_op_debug_desc_t buildOpDebug = { ZE_STRUCTURE_TYPE_RTAS_BUILDER_BUILD_OP_DEBUG_DESC };
     buildOpDebug.dispatchGlobalsPtr = dispatchGlobalsPtr;
     args.pNext = &buildOpDebug;
 #endif
@@ -1363,7 +1364,7 @@ struct Scene
 
   std::shared_ptr<Geometry> operator[] ( size_t i ) { return geometries[i]; }
 
-  typedef sycl::usm_allocator<std::shared_ptr<Geometry>, sycl::usm::alloc::shared> geometries_alloc_ty;
+  typedef usm_allocator<std::shared_ptr<Geometry>, sycl::usm::alloc::shared> geometries_alloc_ty;
   geometries_alloc_ty geometries_alloc;
   std::vector<std::shared_ptr<Geometry>, geometries_alloc_ty> geometries;
 
@@ -2066,12 +2067,7 @@ int main(int argc, char* argv[]) try
   TestType test = TestType::TRIANGLES_COMMITTED_HIT;
   InstancingType inst = InstancingType::NONE;
   BuildMode buildMode = BuildMode::BUILD_EXPECTED_SIZE;
-
-#if defined(EMBREE_SYCL_L0_RTAS_BUILDER)
   ZeWrapper::RTAS_BUILD_MODE rtas_build_mode = ZeWrapper::RTAS_BUILD_MODE::LEVEL_ZERO;
-#else
-  ZeWrapper::RTAS_BUILD_MODE rtas_build_mode = ZeWrapper::RTAS_BUILD_MODE::INTERNAL;
-#endif
   
   bool jit_cache = false;
   uint32_t numThreads = tbb::this_task_arena::max_concurrency();
@@ -2091,9 +2087,6 @@ int main(int argc, char* argv[]) try
     else if (strcmp(argv[i], "--level-zero-rtas-builder") == 0) {
       rtas_build_mode = ZeWrapper::RTAS_BUILD_MODE::LEVEL_ZERO;
     }
-    else if (strcmp(argv[i], "--default-rtas-builder") == 0) {
-      rtas_build_mode = ZeWrapper::RTAS_BUILD_MODE::AUTO;
-    }
     else if (strcmp(argv[i], "--triangles-committed-hit") == 0) {
       test = TestType::TRIANGLES_COMMITTED_HIT;
     }
@@ -2156,6 +2149,11 @@ int main(int argc, char* argv[]) try
     }
   }
 
+  if (rtas_build_mode == ZeWrapper::INTERNAL)
+    std::cout << "using internal RTAS builder (ZE_experimental_rtas_builder)" << std::endl;
+  else
+    std::cout << "using Level Zero RTAS builder (ZE_experimental_rtas_builder)" << std::endl;
+
   if (jit_cache)
     std::cout << "WARNING: JIT caching is not supported!" << std::endl;
 
@@ -2191,35 +2189,42 @@ int main(int argc, char* argv[]) try
   ze_result_t result = ZE_RESULT_SUCCESS;
   sycl::platform platform = device.get_platform();
   ze_driver_handle_t hDriver = sycl::get_native<sycl::backend::ext_oneapi_level_zero>(platform);
+  ze_device_handle_t hDevice = sycl::get_native<sycl::backend::ext_oneapi_level_zero>(device);
 
-  /* enable RTAS extension only when enabled */
-  if (rtas_build_mode == ZeWrapper::RTAS_BUILD_MODE::AUTO)
+  /* check if ray tracing hardware is supported */
+  ze_device_raytracing_ext_properties_t raytracing_properties = { ZE_STRUCTURE_TYPE_DEVICE_RAYTRACING_EXT_PROPERTIES };
+  ze_device_module_properties_t module_properties = { ZE_STRUCTURE_TYPE_DEVICE_MODULE_PROPERTIES, &raytracing_properties };
+  result = ZeWrapper::zeDeviceGetModuleProperties(hDevice, &module_properties);
+  if (result != ZE_RESULT_SUCCESS)
+    throw std::runtime_error("zeDeviceGetModuleProperties failed");
+  
+  const bool rayQuerySupported = raytracing_properties.flags & ZE_DEVICE_RAYTRACING_EXT_FLAG_RAYQUERY;
+  if (!rayQuerySupported)
+    throw std::runtime_error("Device does not support ray tracing");
+
+  /* check if RTAS extension available */
+  uint32_t count = 0;
+  std::vector<ze_driver_extension_properties_t> extensions;
+  result = ZeWrapper::zeDriverGetExtensionProperties(hDriver,&count,extensions.data());
+  if (result != ZE_RESULT_SUCCESS)
+    throw std::runtime_error("zeDriverGetExtensionProperties failed");
+  
+  extensions.resize(count);
+  result = ZeWrapper::zeDriverGetExtensionProperties(hDriver,&count,extensions.data());
+  if (result != ZE_RESULT_SUCCESS)
+    throw std::runtime_error("zeDriverGetExtensionProperties failed");
+  
+  bool ze_rtas_builder = false;
+  for (uint32_t i=0; i<extensions.size(); i++)
   {
-    uint32_t count = 0;
-    std::vector<ze_driver_extension_properties_t> extensions;
-    result = ZeWrapper::zeDriverGetExtensionProperties(hDriver,&count,extensions.data());
-    if (result != ZE_RESULT_SUCCESS)
-      throw std::runtime_error("zeDriverGetExtensionProperties failed");
-    
-    extensions.resize(count);
-    result = ZeWrapper::zeDriverGetExtensionProperties(hDriver,&count,extensions.data());
-    if (result != ZE_RESULT_SUCCESS)
-      throw std::runtime_error("zeDriverGetExtensionProperties failed");
-    
-    bool ze_rtas_builder = false;
-    for (uint32_t i=0; i<extensions.size(); i++)
-    {
-      if (strncmp("ZE_experimental_rtas_builder",extensions[i].name,sizeof(extensions[i].name)) == 0)
-        ze_rtas_builder = true;
-    }
-
-    if (ze_rtas_builder)
-      result = ZeWrapper::initRTASBuilder(hDriver,ZeWrapper::RTAS_BUILD_MODE::AUTO);
-    else
-      result = ZeWrapper::initRTASBuilder(hDriver,ZeWrapper::RTAS_BUILD_MODE::INTERNAL);
+    if (strncmp("ZE_experimental_rtas_builder",extensions[i].name,sizeof(extensions[i].name)) == 0)
+      ze_rtas_builder = true;
   }
-  else
-    result = ZeWrapper::initRTASBuilder(hDriver,rtas_build_mode);
+  if (!ze_rtas_builder)
+    throw std::runtime_error("ZE_experimental_rtas_builder extension not available");
+
+  /* initialize RTAS builder extension in ZeWrapper */
+  result = ZeWrapper::initRTASBuilder(EXP_API,hDriver,rtas_build_mode);
 
   if (result == ZE_RESULT_ERROR_DEPENDENCY_UNAVAILABLE)
     throw std::runtime_error("cannot load ZE_experimental_rtas_builder extension");
@@ -2227,13 +2232,7 @@ int main(int argc, char* argv[]) try
   if (result != ZE_RESULT_SUCCESS)
     throw std::runtime_error("cannot initialize ZE_experimental_rtas_builder extension");
   
-  if (ZeWrapper::rtas_builder == ZeWrapper::INTERNAL)
-    std::cout << "using internal RTAS builder" << std::endl;
-  else
-    std::cout << "using Level Zero RTAS builder" << std::endl;
-
   /* get acceleration structure format for this device */
-  ze_device_handle_t  hDevice  = sycl::get_native<sycl::backend::ext_oneapi_level_zero>(device);
   ze_rtas_device_exp_properties_t rtasProp = { ZE_STRUCTURE_TYPE_RTAS_DEVICE_EXP_PROPERTIES };
   ze_device_properties_t devProp = { ZE_STRUCTURE_TYPE_DEVICE_PROPERTIES, &rtasProp };
   ze_result_t err = ZeWrapper::zeDeviceGetProperties(hDevice, &devProp );
@@ -2277,6 +2276,9 @@ int main(int argc, char* argv[]) try
 #if defined(ZE_RAYTRACING_RT_SIMULATION)
   RTCore::Cleanup();
 #endif
+
+  if (numErrors == 0) std::cout << "[PASSED]" << std::endl;
+  else                std::cout << "[FAILED]" << std::endl;
   
   return numErrors ? 1 : 0;
 }
